#!/usr/bin/env python
import unittest
import lief
import tempfile
import sys
import subprocess
import stat
import os
import logging
import random

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestAbstract(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    @staticmethod
    def get_abstract_binary(binary):
        return binary.abstract

    @staticmethod
    def get_raw(path):
        raw = None
        with open(path, "rb") as f:
            raw = f.read()
        return list(raw)

    def test_endianness(self):
        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('ELF/ELF32_x86_binary_ls.bin')))
        header = binary.header

        self.assertEqual(header.endianness, lief.ENDIANNESS.LITTLE)


        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('MachO/MachO64_x86-64_binary_id.bin')))
        header = binary.header

        self.assertEqual(header.endianness, lief.ENDIANNESS.LITTLE)


        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('PE/PE64_x86-64_binary_ConsoleApplication1.exe')))
        header = binary.header

        self.assertEqual(header.endianness, lief.ENDIANNESS.LITTLE)


    def test_format(self):
        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('ELF/ELF32_x86_binary_ls.bin')))
        self.assertEqual(binary.format, lief.EXE_FORMATS.ELF)


        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('MachO/MachO64_x86-64_binary_id.bin')))
        self.assertEqual(binary.format, lief.EXE_FORMATS.MACHO)


        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('PE/PE64_x86-64_binary_ConsoleApplication1.exe')))
        self.assertEqual(binary.format, lief.EXE_FORMATS.PE)

    def test_pie(self):
        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('ELF/ELF32_ARM_binary-pie_ls.bin')))
        self.assertEqual(binary.is_pie, True)

        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('MachO/MachO64_x86-64_binary_nm.bin')))
        self.assertEqual(binary.is_pie, True)

        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('PE/PE32_x86_binary_cmd.exe')))
        self.assertEqual(binary.is_pie, True)

        binary = TestAbstract.get_abstract_binary(lief.parse(get_sample('ELF/ELF64_x86-64_binary_ls.bin')))
        self.assertEqual(binary.is_pie, False)


    #def test_parser(self):
    #    binary = lief.parse(TestAbstract.get_raw(get_sample('ELF/ELF32_x86_binary_ls.bin')))
    #    self.assertTrue(isinstance(binary, lief.ELF.Binary))


    #    binary = lief.parse(TestAbstract.get_raw(get_sample('MachO/MachO64_x86-64_binary_id.bin')))
    #    self.assertTrue(isinstance(binary, lief.MachO.Binary))

    #    binary = lief.parse(TestAbstract.get_raw(get_sample('MachO/FAT_MachO_x86_x86-64_library_libc.dylib')))
    #    self.assertTrue(isinstance(binary, lief.MachO.Binary))


    #    binary = lief.parse(TestAbstract.get_raw(get_sample('PE/PE64_x86-64_binary_ConsoleApplication1.exe')))
    #    self.assertTrue(isinstance(binary, lief.PE.Binary))


if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python2
#-*- coding: utf-8 -*-

# This file is used to generate config file for LIEF test
# It uses `Macholib`(https://pypi.python.org/pypi/macholib/) to generate yaml config file

import re
import yaml
import hashlib
from macholib.MachO import MachO
import sys


macho   = MachO(sys.argv[1])
# Not FAT for now
assert(len(macho.headers) == 1)

binary = dict()

header = dict()

m_header = macho.headers[0]
header["magic"]      = m_header.MH_MAGIC
header["cputype"]    = m_header.header.cputype
header["cpusubtype"] = m_header.header.cpusubtype
header["filetype"]   = m_header.header.filetype
header["ncmds"]      = m_header.header.ncmds
header["sizeofcmds"] = m_header.header.sizeofcmds
header["flags"]      = m_header.header.flags
header["reserved"]   = m_header.header.reserved

binary["header"] = header

for lc, cmd, data in m_header.commands:
    print lc.get_cmd_name()
    print cmd.describe()
    print len(data)


print header



#!/usr/bin/env python
import unittest
import lief
import tempfile
import sys
import subprocess
import stat
import os
import logging
import random
import itertools

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.WARNING)
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample


class TestNotes(TestCase):
    LOGGER = logging.getLogger(__name__)

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    @staticmethod
    def safe_delete(output):
        if os.path.isfile(output):
            try:
                os.remove(output)
                return True
            except Exception as e:
               TestNotes.LOGGER.error("Can't delete {} ({})".format(output, e))
               return False


    def test_change_note(self):
        _, output = tempfile.mkstemp(prefix="change_note_")

        etterlog = lief.parse(get_sample('ELF/ELF64_x86-64_binary_etterlog.bin'))
        build_id = etterlog[lief.ELF.NOTE_TYPES.BUILD_ID]

        new_desc = [i & 0xFF for i in range(500)]
        build_id.description = new_desc

        etterlog.write(output)

        etterlog_updated = lief.parse(output)

        self.assertEqual(etterlog[lief.ELF.NOTE_TYPES.BUILD_ID], etterlog_updated[lief.ELF.NOTE_TYPES.BUILD_ID])
        self.safe_delete(output)


    def test_remove_note(self):
        _, output = tempfile.mkstemp(prefix="remove_note_")
        self.logger.info("Output will be: {}".format(output))

        etterlog = lief.parse(get_sample('ELF/ELF64_x86-64_binary_etterlog.bin'))

        build_id = etterlog[lief.ELF.NOTE_TYPES.BUILD_ID]

        etterlog -= build_id

        etterlog.write(output)

        etterlog_updated = lief.parse(output)

        self.assertNotIn(lief.ELF.NOTE_TYPES.BUILD_ID, etterlog_updated)

        self.safe_delete(output)

    def test_add_note(self):
        _, output = tempfile.mkstemp(prefix="add_note_")
        self.logger.info("Output will be: {}".format(output))

        etterlog = lief.parse(get_sample('ELF/ELF64_x86-64_binary_etterlog.bin'))
        note = lief.ELF.Note("Foo", lief.ELF.NOTE_TYPES.GOLD_VERSION, [123])

        etterlog += note

        etterlog.write(output)

        etterlog_updated = lief.parse(output)

        self.assertIn(lief.ELF.NOTE_TYPES.GOLD_VERSION, etterlog_updated)

        self.safe_delete(output)


    def test_android_note(self):
        _, output = tempfile.mkstemp(prefix="android_note_")
        self.logger.info("Output will be: {}".format(output))

        ndkr16 = lief.parse(get_sample('ELF/ELF64_AArch64_piebinary_ndkr16.bin'))
        note = ndkr16.get(lief.ELF.NOTE_TYPES.ABI_TAG)

        self.assertEqual(note.sdk_version, 21)
        self.assertEqual(note.ndk_version[:4], "r16b")
        self.assertEqual(note.ndk_build_number[:7], "4479499")

        note.sdk_version = 15
        note.ndk_version = "r15c"
        note.ndk_build_number = "123456"

        note = ndkr16.get(lief.ELF.NOTE_TYPES.ABI_TAG)

        self.assertEqual(note.sdk_version, 15)
        self.assertEqual(note.ndk_version[:4], "r15c")
        self.assertEqual(note.ndk_build_number[:6], "123456")

        ndkr16.write(output)

        ndkr15 = lief.parse(output)

        note = ndkr15.get(lief.ELF.NOTE_TYPES.ABI_TAG)

        self.assertEqual(note.sdk_version, 15)
        self.assertEqual(note.ndk_version[:4], "r15c")
        self.assertEqual(note.ndk_build_number[:6], "123456")

        self.safe_delete(output)




if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)



#!/usr/bin/env python
import unittest
import lief
import logging
import pprint
import json
import os

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.FATAL)
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

class TestDEX(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)


class TestDEX35(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)

        self.kik_dex35 = lief.DEX.parse(get_sample("DEX/DEX35_kik.android.12.8.0.dex"))
        self.sb_dex35 = lief.DEX.parse(get_sample("DEX/DEX35_com.starbucks.mobilecard.dex"))

    def test_kik_header(self):
        header = self.kik_dex35.header

        self.assertEqual(header.magic, [100, 101, 120, 10, 48, 51, 53, 0])
        self.assertEqual(header.checksum, 0x5eabacd)
        self.assertEqual(header.signature, [222, 148, 89, 234, 112, 212, 217, 127, 146, 201, 101, 115, 66, 163, 44, 125, 125, 142, 208, 242])
        self.assertEqual(header.file_size, 0x78ac64)
        self.assertEqual(header.header_size, 0x70)
        self.assertEqual(header.endian_tag, 0x12345678)
        self.assertEqual(header.map_offset, 0x78ab88)
        self.assertEqual(header.strings, (0x70, 51568))
        self.assertEqual(header.link, (0, 0))
        self.assertEqual(header.types, (0x32630, 12530))
        self.assertEqual(header.prototypes, (0x3e9f8, 14734))
        self.assertEqual(header.fields, (0x69ca0, 33376))
        self.assertEqual(header.methods, (0xaafa0, 65254))
        self.assertEqual(header.classes, (0x12a6d0, 6893))
        self.assertEqual(header.data, (0x160470, 6465524))

    def test_kik_map(self):
        dex_map = self.kik_dex35.map
        #print(dex_map)

    def test_kik_class(self):
        classes = self.kik_dex35.classes
        self.assertEqual(len(classes), 12123)

        c0 = self.kik_dex35.get_class("android.graphics.drawable.ShapeDrawable")
        self.assertEqual(c0.pretty_name, "android.graphics.drawable.ShapeDrawable")
        self.assertEqual(len(c0.methods), 3)

        cls = self.kik_dex35.get_class("com.kik.video.mobile.KikVideoService$JoinConvoConferenceResponse$Result")
        self.assertEqual(cls.access_flags, [
            lief.DEX.ACCESS_FLAGS.PUBLIC,
            lief.DEX.ACCESS_FLAGS.FINAL,
            lief.DEX.ACCESS_FLAGS.ENUM])

        self.assertEqual(cls.source_filename, "SourceFile")
        self.assertEqual(cls.package_name, "com/kik/video/mobile")
        self.assertEqual(cls.name, "KikVideoService$JoinConvoConferenceResponse$Result")
        self.assertEqual(cls.parent.pretty_name, "java.lang.Enum")
        self.assertEqual(len(cls.methods), 14)
        self.assertEqual(cls.index, 6220)

        methods_name = set(m.name for m in cls.methods)
        self.assertEqual(methods_name, set([
            '<clinit>', '<init>', 'forNumber', 'getDescriptor',
            'internalGetValueMap', 'valueOf', 'valueOf',
            'valueOf', 'values', 'getDescriptorForType',
            'getNumber', 'getValueDescriptor',
            'clone', 'ordinal']))


    def test_kik_methods(self):
        methods = self.kik_dex35.methods

        self.assertEqual(len(methods), 65254)

        ValueAnimator = self.kik_dex35.get_class("android.animation.ValueAnimator")
        m0 = ValueAnimator.get_method("setRepeatMode")[0]

        self.assertEqual(m0.name,            "setRepeatMode")
        self.assertEqual(m0.cls.pretty_name, "android.animation.ValueAnimator")
        self.assertEqual(m0.code_offset,     0)
        self.assertEqual(m0.bytecode,        [])
        self.assertEqual(m0.index,           100)
        #self.assertEqual(m0.is_virtual,      False) # TODO
        self.assertEqual(m0.prototype.return_type.value, lief.DEX.Type.PRIMITIVES.VOID_T)
        self.assertEqual(m0.access_flags, [])





class TestDEX37(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)

class TestDEX38(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        #self.dex38 = lief.DEX.parse(get_sample("DEX/DEX38-Framework.dex"))
        #self.dex38 = lief.DEX.parse("/home/romain/dev/LIEF/_work/dex/ArrayClass.dex")

    def test_header(self):
        #print(self.dex38.header)
        pass




if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
import lief
from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.INFO)

from subprocess import Popen
from unittest import TestCase
from utils import get_sample

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))
STUB = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))

LIBADD_C = """\
#include <stdlib.h>
#include <stdio.h>

int add(int a, int b);

int add(int a, int b) {
  printf("%d + %d = %d\\n", a, b, a + b);
  return a + b;
}
"""

BINADD_C = """\
#include <stdio.h>
#include <stdlib.h>

int add(int a, int b);

int main(int argc, char **argv) {
  if (argc != 3) {
    printf("Usage: %s <a> <b>\\n", argv[0]);
    exit(-1);
  }

  int res = add(atoi(argv[1]), atoi(argv[2]));
  printf("From myLIb, a + b = %d\\n", res);
  return 0;
}
"""
class LibAddSample(object):
    COUNT = 0
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_sample_{:d}'.format(LibAddSample.COUNT))
        self.logger.debug("temp dir: {}".format(self.tmp_dir))

        LibAddSample.COUNT += 1

        self.binadd_path = os.path.join(self.tmp_dir, "binadd.c")
        self.libadd_path = os.path.join(self.tmp_dir, "libadd.c")

        self.libadd_so  = os.path.join(self.tmp_dir, "libadd.so")
        self.binadd_bin = os.path.join(self.tmp_dir, "binadd.bin")

        self.compiler = '/usr/bin/cc'

        if self.compiler is None:
            self.logger.error("Unable to find a compiler")
            sys.exit(0)

        self.logger.debug("Compiler: {}".format(self.compiler))

        with open(self.binadd_path, 'w') as f:
            f.write(BINADD_C)

        with open(self.libadd_path, 'w') as f:
            f.write(LIBADD_C)

        self._compile_libadd()
        self._compile_binadd()


    def _compile_libadd(self, extra_flags=[]):
        if os.path.isfile(self.libadd_so):
            os.remove(self.libadd_so)

        CC_FLAGS = ['-fPIC', '-shared', '-Wl,-soname,libadd.so'] + extra_flags
        cmd = [self.compiler, '-o', self.libadd_so] + CC_FLAGS + [self.libadd_path]
        self.logger.debug("Compile 'libadd' with: {}".format(" ".join(cmd)))

        p = Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

        stdout, _ = p.communicate()

        self.logger.debug(stdout)


    def _compile_binadd(self, extra_flags=[]):
        if os.path.isfile(self.binadd_bin):
            os.remove(self.binadd_bin)

        CC_FLAGS = ['-L', self.tmp_dir] + extra_flags
        cmd = [self.compiler, '-o', self.binadd_bin] + CC_FLAGS + [self.binadd_path, '-ladd']
        self.logger.debug("Compile 'binadd' with: {}".format(" ".join(cmd)))
        p = Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout)

    @property
    def libadd(self):
        return self.libadd_so

    @property
    def binadd(self):
        return self.binadd_bin

    @property
    def directory(self):
        return self.tmp_dir

    def remove(self):
        if os.path.isdir(self.directory):
            shutil.rmtree(self.directory)

    def __del__(self):
        self.remove()


class TestDynamic(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_remove_library(self):
        sample = LibAddSample()
        libadd = lief.parse(sample.libadd)
        binadd = lief.parse(sample.binadd)

        libadd_needed = binadd.get_library("libadd.so")
        binadd -= libadd_needed
        self.assertFalse(binadd.has_library("libadd.so"))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_remove_tag(self):
        sample = LibAddSample()
        libadd = lief.parse(sample.libadd)
        binadd = lief.parse(sample.binadd)
        self.logger.debug("BEFORE")
        list(map(lambda e : self.logger.debug(e), binadd.dynamic_entries))
        self.logger.debug("")
        binadd -= lief.ELF.DYNAMIC_TAGS.NEEDED

        self.logger.debug("AFTER")
        list(map(lambda e : self.logger.debug(e), binadd.dynamic_entries))
        self.logger.debug("")

        self.assertTrue(all(e.tag != lief.ELF.DYNAMIC_TAGS.NEEDED for e in binadd.dynamic_entries))

    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_runpath_api(self):
        sample = LibAddSample()
        libadd = lief.parse(sample.libadd)
        binadd = lief.parse(sample.binadd)

        rpath = lief.ELF.DynamicEntryRunPath()
        rpath = binadd.add(rpath)
        self.logger.debug(rpath)
        rpath += "/tmp"

        self.logger.debug(rpath)

        self.assertEqual(rpath.paths, ["/tmp"])
        self.assertEqual(rpath.runpath, "/tmp")

        rpath.insert(0, "/foo")

        self.assertEqual(rpath.paths, ["/foo", "/tmp"])
        self.assertEqual(rpath.runpath, "/foo:/tmp")

        rpath.paths = ["/foo", "/tmp", "/bar"]

        self.logger.debug(rpath)

        self.assertEqual(rpath.paths, ["/foo", "/tmp", "/bar"])
        self.assertEqual(rpath.runpath, "/foo:/tmp:/bar")

        rpath -= "/tmp"

        self.logger.debug(rpath)

        self.assertEqual(rpath.runpath, "/foo:/bar")

        rpath.remove("/foo").remove("/bar")

        self.logger.debug(rpath)

        self.assertEqual(rpath.runpath, "")

        self.logger.debug(rpath)



    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_change_libname(self):
        sample = LibAddSample()
        libadd = lief.parse(sample.libadd)
        binadd = lief.parse(sample.binadd)

        new_name = "libwhichhasalongverylongname.so"


        self.assertIn(lief.ELF.DYNAMIC_TAGS.SONAME, libadd)

        so_name = libadd[lief.ELF.DYNAMIC_TAGS.SONAME]
        self.logger.debug("DT_SONAME: {}".format(so_name.name))
        so_name.name = new_name

        libfoo_path = os.path.join(sample.directory, new_name)
        self.logger.debug(libfoo_path)
        libadd.write(libfoo_path)

        libfoo = lief.parse(libfoo_path)

        new_so_name = libadd[lief.ELF.DYNAMIC_TAGS.SONAME]
        # Check builder did the job right
        self.assertEqual(new_so_name.name, new_name)

        libadd_needed = binadd.get_library("libadd.so")
        libadd_needed.name = new_name

        # Add a RPATH entry
        rpath = lief.ELF.DynamicEntryRunPath(sample.directory)
        rpath = binadd.add(rpath)
        self.logger.debug(rpath)

        new_binadd_path = os.path.join(sample.directory, "binadd_updated.bin")
        self.logger.debug(new_binadd_path)
        binadd.write(new_binadd_path)

        # Remove original binaries:
        os.remove(sample.libadd)
        os.remove(sample.binadd)

        # Run the new executable
        st = os.stat(libfoo_path)
        os.chmod(libfoo_path, st.st_mode | stat.S_IEXEC)

        st = os.stat(new_binadd_path)
        os.chmod(new_binadd_path, st.st_mode | stat.S_IEXEC)

        p = Popen([new_binadd_path, '1', '2'],
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT)
                #env={"LD_LIBRARY_PATH": sample.directory}) # Shouldn't be needed if RPATH inject succeed
        stdout, _ = p.communicate()
        if p.returncode > 0:
            self.logger.fatal(stdout.decode("utf8"))
            self.assertEqual(p.returncode, 0)

        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'From myLIb, a \+ b = 3', stdout.decode("utf8")))

        sample.remove()



if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import lief
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
import time
import ctypes
import zipfile

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestBuilder(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_builder')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))

        if sys.platform.startswith("win"):
            SEM_NOGPFAULTERRORBOX = 0x0002 # From MSDN
            ctypes.windll.kernel32.SetErrorMode(SEM_NOGPFAULTERRORBOX);

    def test_add_multiples_sections(self):
        sample_file = get_sample('PE/PE32_x86_binary_Notepad++.zip')
        sample_dir  = os.path.join(self.tmp_dir, "Notepad++")

        sample = os.path.join(sample_dir, "notepad++.exe")
        output = os.path.join(sample_dir, "notepad++_sections.exe")

        zip_ref = zipfile.ZipFile(sample_file, 'r')
        zip_ref.extractall(self.tmp_dir)
        zip_ref.close()

        notepadpp = lief.parse(sample)

        # Add 20 sections to the binary
        for i in range(20):
            section = lief.PE.Section(".section_{}".format(i))
            section.content = [i & 0xFF for i in range(0x200)]
            notepadpp.add_section(section)

        builder = lief.PE.Builder(notepadpp)
        builder.build()

        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "notepad++_sections.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)


    def test_imports_notepadpp(self):
        sample_file = get_sample('PE/PE32_x86_binary_Notepad++.zip')
        sample_dir  = os.path.join(self.tmp_dir, "Notepad++")

        sample = os.path.join(sample_dir, "notepad++.exe")
        output = os.path.join(sample_dir, "notepad++_imports.exe")

        zip_ref = zipfile.ZipFile(sample_file, 'r')
        zip_ref.extractall(self.tmp_dir)
        zip_ref.close()

        notepadpp = lief.parse(sample)

        # Disable ASLR
        notepadpp.optional_header.dll_characteristics &= ~lief.PE.DLL_CHARACTERISTICS.DYNAMIC_BASE

        # Disable NX protection
        notepadpp.optional_header.dll_characteristics &= ~lief.PE.DLL_CHARACTERISTICS.NX_COMPAT

        builder = lief.PE.Builder(notepadpp)
        builder.build_imports(True).patch_imports(True)
        builder.build()

        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "notepad++_imports.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)

    def tearDown(self):
        # Delete it
        try:
            if os.path.isdir(self.tmp_dir):
                shutil.rmtree(self.tmp_dir)
        except Exception as e:
            self.logger.error(e)


if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)



#!/usr/bin/env python
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
from subprocess import Popen

import lief
from lief.ELF import Segment
from lief import Logger
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))

class TestAddSegment(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_add_segment')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_simple(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_ls.bin')
        stub        = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        output      = os.path.join(self.tmp_dir, "ls.replace_segment")
        target      = lief.parse(sample_path)


        if not lief.ELF.SEGMENT_TYPES.NOTE in target:
            self.logger.error("Note not found!")
            return

        segment                 = stub.segments[0]
        original_va             = segment.virtual_address
        segment.virtual_address = 0
        segment                 = target.replace(segment, target[lief.ELF.SEGMENT_TYPES.NOTE])
        new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

        target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_gcc(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_gcc.bin')
        stub        = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        output      = os.path.join(self.tmp_dir, "gcc.replace_segment")
        target      = lief.parse(sample_path)


        if not lief.ELF.SEGMENT_TYPES.NOTE in target:
            self.logger.error("Note not found!")
            return

        segment                 = stub.segments[0]
        original_va             = segment.virtual_address
        segment.virtual_address = 0
        segment                 = target.replace(segment, target[lief.ELF.SEGMENT_TYPES.NOTE])
        new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

        target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_ssh(self):
        stub        = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        output      = os.path.join(self.tmp_dir, "ssh.replace_segment")
        target      = lief.parse("/usr/bin/ssh")

        if not lief.ELF.SEGMENT_TYPES.NOTE in target:
            self.logger.error("Note not found!")
            return

        segment                 = stub.segments[0]
        original_va             = segment.virtual_address
        segment.virtual_address = 0
        segment                 = target.replace(segment, target[lief.ELF.SEGMENT_TYPES.NOTE])
        new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

        target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))

    def tearDown(self):
        # Delete it
        if os.path.isdir(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import sys
import os

def is_elf(file):
    magic = None
    with open(file, 'rb') as f:
        raw = f.read()
        magic = raw[:4]
    return magic[0] == 0x7F and \
           magic[1] == ord('E') and \
           magic[2] == ord('L') and \
           magic[3] == ord('F')

def is_pe(file):
    magic = None
    with open(file, 'rb') as f:
        raw = f.read()
        magic = raw[:2]
    return magic[0] == ord('M') and \
           magic[1] == ord('Z')

def is_macho(file):
    magic = None
    with open(file, 'rb') as f:
        raw = f.read()
        magic = raw[:4]
    magic = list(magic)

    magics = [
            [0xFE, 0xED, 0xFA, 0xCE],
            [0xCE, 0xFA, 0xED, 0xFE],
            [0xFE, 0xED, 0xFA, 0xCF],
            [0xCF, 0xFA, 0xED, 0xFE],
            [0xCA, 0xFE, 0xBA, 0xBE],
            [0xBE, 0xBA, 0xFE, 0xCA],
            ]
    return any(m == magic for m in magics)

def clean(directory):
    whitelist = [".git"]
    for dirname, subdir, files in os.walk(directory):

        if any(d in dirname for d in whitelist):
            continue

        for f in files:
            fullpath = os.path.join(dirname, f)

            if not (is_elf(fullpath)  or is_pe(fullpath) or is_macho(fullpath)):
                print("Removing '{}'".format(fullpath))
                try:
                    os.remove(fullpath)
                except Exception as e:
                    print("Error: {}".format(e))


def main():
    if len(sys.argv) != 2:
        print("Usage: {} <corpus>".format(sys.argv[0]))
        return 1
    clean(sys.argv[1])
    return 0

if __name__ == "__main__":
    main()






#!/usr/bin/env python
# -*- coding: utf-8 -*-

import lief
import unittest
import logging
import tempfile
import shutil
import os
import sys
import stat
import subprocess
import time
import ctypes
import zipfile

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestResource(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_resource')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))


    def test_change_icons(self):
        mfc_path = get_sample('PE/PE64_x86-64_binary_mfc-application.exe')
        mfc = lief.parse(mfc_path)
        mfc_resources_manger = mfc.resources_manager

        cmd_path = get_sample('PE/PE64_x86-64_binary_cmd.exe')
        cmd = lief.parse(cmd_path)
        cmd_resources_manger = cmd.resources_manager

        if not mfc_resources_manger.has_icons:
            print("'{}' has no manifest. Abort!".format(mfc.name))
            sys.exit(1)

        if not cmd_resources_manger.has_icons:
            print("'{}' has no manifest. Abort!".format(mfc.name))
            sys.exit(1)

        mfc_icons = mfc_resources_manger.icons
        cmd_icons = cmd_resources_manger.icons

        for i in range(min(len(mfc_icons), len(cmd_icons))):
            mfc_resources_manger.change_icon(mfc_icons[i], cmd_icons[i])


        output = os.path.join(self.tmp_dir, "mfc_test_change_icon.exe")
        builder = lief.PE.Builder(mfc)
        builder.build_resources(True)
        builder.build()
        builder.write(output)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "mfc_test_change_icon.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)



    def test_resource_version(self):
        sample_path = get_sample('PE/PE64_x86-64_binary_mfc-application.exe')
        mfc = lief.parse(sample_path)
        resources_manger = mfc.resources_manager

        self.assertEqual(resources_manger.has_version, True)
        version = resources_manger.version

        self.assertEqual(version.type, 0)
        self.assertEqual(version.key, 'VS_VERSION_INFO')

        self.assertEqual(version.has_string_file_info, True)
        self.assertEqual(version.has_var_file_info,    True)
        self.assertEqual(version.has_fixed_file_info,  True)

        fixed_file_info  = version.fixed_file_info
        string_file_info = version.string_file_info
        var_file_info    = version.var_file_info

        # Check ResourceFixedFileInfo
        self.assertEqual(fixed_file_info.signature, 0xFEEF04BD)
        self.assertEqual(fixed_file_info.struct_version, 0x10000)
        self.assertEqual(fixed_file_info.file_version_MS, 0x010000)
        self.assertEqual(fixed_file_info.file_version_LS, 0x000001)
        self.assertEqual(fixed_file_info.product_version_MS, 0x010000)
        self.assertEqual(fixed_file_info.product_version_LS, 0x000001)
        self.assertEqual(fixed_file_info.file_flags_mask, 63)
        self.assertEqual(fixed_file_info.file_flags, 0)
        self.assertEqual(fixed_file_info.file_os, lief.PE.FIXED_VERSION_OS.NT_WINDOWS32)
        self.assertEqual(fixed_file_info.file_type, lief.PE.FIXED_VERSION_FILE_TYPES.APP)
        self.assertEqual(fixed_file_info.file_subtype, lief.PE.FIXED_VERSION_FILE_SUB_TYPES.UNKNOWN)
        self.assertEqual(fixed_file_info.file_date_MS, 0)
        self.assertEqual(fixed_file_info.file_date_LS, 0)

        # Check ResourceStringFileInfo
        self.assertEqual(string_file_info.type, 1)
        self.assertEqual(string_file_info.key, "StringFileInfo")
        self.assertEqual(len(string_file_info.langcode_items), 1)

        langcode_item = string_file_info.langcode_items[0]
        self.assertEqual(langcode_item.type, 1)
        self.assertEqual(langcode_item.lang, lief.PE.RESOURCE_LANGS.FRENCH)
        self.assertEqual(langcode_item.sublang, lief.PE.RESOURCE_SUBLANGS.FRENCH)
        self.assertEqual(langcode_item.code_page, lief.PE.CODE_PAGES.UTF_16)
        items = langcode_item.items
        self.assertIn('CompanyName', items)
        if sys.version_info < (3,):
            self.assertEqual(items['CompanyName'].encode("utf8"), 'TODO: <Nom de la société>')
        else:
            self.assertEqual(items['CompanyName'], 'TODO: <Nom de la société>')

        self.assertIn('FileVersion', items)
        self.assertEqual(items['FileVersion'], '1.0.0.1')

        self.assertIn('InternalName', items)
        self.assertEqual(items['InternalName'], 'Hello.exe')

        self.assertIn('LegalCopyright', items)
        if sys.version_info < (3,):
            self.assertEqual(items['LegalCopyright'].encode("utf8"), 'TODO: (c) <Nom de la société>.  Tous droits réservés.')
        else:
            self.assertEqual(items['LegalCopyright'], 'TODO: (c) <Nom de la société>.  Tous droits réservés.')

        self.assertIn('OriginalFilename', items)
        self.assertEqual(items['OriginalFilename'], 'Hello.exe')

        self.assertIn('ProductName', items)
        self.assertEqual(items['ProductName'], 'TODO: <Nom du produit>')

        self.assertIn('ProductVersion', items)
        self.assertEqual(items['ProductVersion'], '1.0.0.1')

        # Check ResourceVarFileInfo
        self.assertEqual(var_file_info.type, 1)
        self.assertEqual(var_file_info.key, "VarFileInfo")
        self.assertEqual(len(var_file_info.translations), 1)
        self.assertEqual(var_file_info.translations[0], 0x4b0040c)


    def test_mfc_resource_builder(self):
        sample_path = get_sample('PE/PE64_x86-64_binary_mfc-application.exe')
        output      = os.path.join(self.tmp_dir, "mfc_test_rsrc.exe")

        mfc = lief.parse(sample_path)

        builder = lief.PE.Builder(mfc)
        builder.build_resources(True)
        builder.build()
        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "mfc_test_rsrc.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)


    #def test_evince_resource_builder(self):
    #    sample_file = get_sample('PE/PE32_x86_binary_EvincePortable.zip')
    #    sample_dir  = os.path.join(self.tmp_dir, "EvincePortable")

    #    sample = os.path.join(sample_dir, "EvincePortable.exe")
    #    output = os.path.join(sample_dir, "evince_rsrc.exe")

    #    zip_ref = zipfile.ZipFile(sample_file, 'r')
    #    zip_ref.extractall(self.tmp_dir)
    #    zip_ref.close()

    #    evince = lief.parse(sample)

    #    builder = lief.PE.Builder(evince)
    #    builder.build_resources(True)
    #    builder.build()
    #    builder.write(output)

    #    st = os.stat(output)
    #    os.chmod(output, st.st_mode | stat.S_IEXEC)

    #    if sys.platform.startswith("win"):
    #        subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
    #        p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
    #        time.sleep(3)
    #        q = Popen(["taskkill", "/im", "evince_rsrc.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    #        stdout, _ = p.communicate()
    #        self.logger.debug(stdout.decode("utf8"))

    #        stdout, _ = q.communicate()
    #        self.logger.debug(stdout.decode("utf8"))

    #        self.assertEqual(q.returncode, 0)

    def test_notepadpp_resource_builder(self):
        sample_file = get_sample('PE/PE32_x86_binary_Notepad++.zip')
        sample_dir  = os.path.join(self.tmp_dir, "Notepad++")

        sample = os.path.join(sample_dir, "notepad++.exe")
        output = os.path.join(sample_dir, "notepad++_rsrc.exe")

        zip_ref = zipfile.ZipFile(sample_file, 'r')
        zip_ref.extractall(self.tmp_dir)
        zip_ref.close()

        notepadpp = lief.parse(sample)

        builder = lief.PE.Builder(notepadpp)
        builder.build_resources(True)
        builder.build()
        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "notepad++_rsrc.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)


    def test_filezilla_resource_builder(self):
        sample_file = get_sample('PE/PE64_x86-64_binary_FileZilla.zip')
        sample_dir  = os.path.join(self.tmp_dir, "FileZilla")

        sample = os.path.join(sample_dir, "filezilla.exe")
        output = os.path.join(sample_dir, "filezilla_rsrc.exe")

        zip_ref = zipfile.ZipFile(sample_file, 'r')
        zip_ref.extractall(self.tmp_dir)
        zip_ref.close()

        filezilla = lief.parse(sample)

        builder = lief.PE.Builder(filezilla)
        builder.build_resources(True)
        builder.build()
        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "filezilla_rsrc.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)






    def tearDown(self):
        # Delete it
        try:
            if os.path.isdir(self.tmp_dir):
                shutil.rmtree(self.tmp_dir)
        except Exception as e:
            self.logger.error(e)



if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)

